(function (global, factory) {
    if (typeof define === "function" && define.amd) {
        define(['exports', 'react', 'lodash.assign', 'lodash.noop', 'lodash.omit'], factory);
    } else if (typeof exports !== "undefined") {
        factory(exports, require('react'), require('lodash.assign'), require('lodash.noop'), require('lodash.omit'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod.exports, global.react, global.lodash, global.lodash, global.lodash);
        global.ReactHoverObserver = mod.exports;
    }
})(this, function (exports, _react, _lodash, _lodash3, _lodash5) {
    'use strict';

    Object.defineProperty(exports, "__esModule", {
        value: true
    });

    var _react2 = _interopRequireDefault(_react);

    var _lodash2 = _interopRequireDefault(_lodash);

    var _lodash4 = _interopRequireDefault(_lodash3);

    var _lodash6 = _interopRequireDefault(_lodash5);

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    exports.default = _react2.default.createClass({

        displayName: 'ReactHoverObserver',

        getDefaultProps: function getDefaultProps() {
            return {
                hoverDelayInMs: 0,
                hoverOffDelayInMs: 0,
                onMouseEnter: _lodash4.default,
                onMouseLeave: _lodash4.default,
                onMouseOver: _lodash4.default,
                onMouseOut: _lodash4.default
            };
        },


        propTypes: {
            className: _react.PropTypes.string,
            hoverDelayInMs: _react.PropTypes.number,
            hoverOffDelayInMs: _react.PropTypes.number,
            onMouseEnter: _react.PropTypes.func,
            onMouseLeave: _react.PropTypes.func,
            onMouseOver: _react.PropTypes.func,
            onMouseOut: _react.PropTypes.func
        },

        getInitialState: function getInitialState() {
            return {
                isHovering: false
            };
        },
        onMouseEnter: function onMouseEnter(e) {
            this.props.onMouseEnter({
                e: e,
                setIsHovering: this.setIsHovering,
                unsetIsHovering: this.unsetIsHovering
            });
        },
        onMouseLeave: function onMouseLeave(e) {
            this.props.onMouseLeave({
                e: e,
                setIsHovering: this.setIsHovering,
                unsetIsHovering: this.unsetIsHovering
            });
        },
        onMouseOver: function onMouseOver(e) {
            this.props.onMouseOver({
                e: e,
                setIsHovering: this.setIsHovering,
                unsetIsHovering: this.unsetIsHovering
            });
        },
        onMouseOut: function onMouseOut(e) {
            this.props.onMouseOut({
                e: e,
                setIsHovering: this.setIsHovering,
                unsetIsHovering: this.unsetIsHovering
            });
        },
        setIsHovering: function setIsHovering() {
            var _this = this;

            clearTimeout(this.hoverOffScheduleId);

            this.hoverScheduleId = setTimeout(function () {
                _this.setState({
                    isHovering: true
                });
            }, this.props.hoverDelayInMs);
        },
        unsetIsHovering: function unsetIsHovering() {
            var _this2 = this;

            clearTimeout(this.hoverScheduleId);

            this.hoverOffScheduleId = setTimeout(function () {
                _this2.setState({
                    isHovering: false
                });
            }, this.props.hoverOffDelayInMs);
        },
        renderChildrenWithProps: function renderChildrenWithProps(children, props) {
            return _react.Children.map(children, function (child) {
                return (0, _react.cloneElement)(child, props);
            });
        },
        render: function render() {
            var _props = this.props;
            var children = _props.children;
            var className = _props.className;

            var childProps = (0, _lodash2.default)({}, { isHovering: this.state.isHovering }, (0, _lodash6.default)(this.props, ['children', 'className', 'hoverDelayInMs', 'hoverOffDelayInMs', 'onMouseEnter', 'onMouseLeave', 'onMouseOver', 'onMouseOut']));

            return _react2.default.createElement(
                'div',
                {
                    className: className,
                    onMouseEnter: this.onMouseEnter,
                    onMouseLeave: this.onMouseLeave,
                    onMouseOver: this.onMouseOver,
                    onMouseOut: this.onMouseOut
                },
                this.renderChildrenWithProps(children, childProps)
            );
        }
    });
});